from terminal import Terminal
from entity import Character, Player, Item, Enemy
from actions import *
from floor import Floor, TileType

TEXTURES = {
    TileType.WALL: '#',
    TileType.AIR:  '.',
    Player: '@'
}

class Game:
    def __init__(self):
        self.term = Terminal()
        self.player = Player()
        self.floor = Floor(20, 10)
        self.should_exit = False
        self.ticks = 0
        self.schedule = {}

    def run(self):
        """
        Runs the game
        """
        self.instance_character(3, 3, self.player)
        while not self.should_exit:
            self.render()
            self.step()
            self.ticks += 1

    def instance_character(self, x, y, character: Character):
        self.floor.entities.add_creature(x, y, character)
        self.reschedule(character)

    def reschedule(self, character: Character):
        """
        Calculates turns for the next avaliable ticks untill player turn
        """
        time = (action_timing[character.action.__class__] // character.speed) + self.ticks
        if self.schedule.get(time):
            self.schedule[time].append(character)
        else:
            self.schedule[time] = [character]

    def render(self):
        """
        Renders the game
        """
        for x in range(self.floor.width):
            for y in range(self.floor.height):
                if (x, y) in self.floor.entities.pos_creatures:
                    self.term.put_char(x, y, TEXTURES[Player])
                elif (x, y) in self.floor.entities.items:
                    self.term.put_char(x, y, 'i')
                else:
                    self.term.put_char(x, y, TEXTURES[self.floor.get_tile(x, y)])

    def step(self):
        creaturas = self.schedule.get(self.ticks)
        if creaturas:
            for creatura in creaturas:
                if isinstance(creatura, Player):
                    self.get_event()
                elif isinstance(creatura, Enemy):
                    creatura.calculate_action()
                self.reschedule(creatura)
                self.perform(creatura)
            self.schedule.pop(self.ticks)

    def perform(self, character: Character):
        """
        Performs an action for a creature
        """
        if isinstance(character.action, Move):
            pos = self.floor.entities.pop_creature_ref(character)
            if pos:
                self.floor.entities.add_creature(pos[0] + character.action.x, pos[1] + character.action.y, character)


    def get_event(self):
        """
        Waits for a game event to happen
        """
        keycode = self.term.get_key()
        if keycode == "q":
            self.should_exit = True
        elif keycode == "KEY_RIGHT":
            self.player.action = Move(1, 0)
        elif keycode == "KEY_LEFT":
            self.player.action = Move(-1, 0)
        elif keycode == "KEY_DOWN":
            self.player.action = Move(0, 1)
        elif keycode == "KEY_UP":
            self.player.action = Move(0, -1)